@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
+import json
+
 from oauthlib.oauth2 import MetadataEndpoint, Server, TokenEndpoint
 
-import json
 from tests.unittest import TestCase
 
 
@@ -19,8 +20,8 @@ class MetadataEndpointTest(TestCase):
             "introspection_endpoint": "https://foo.bar/introspect",
             "token_endpoint": "https://foo.bar/token"
         }
-        from oauthlib.oauth2 import Server as OAuth2Server
-        from oauthlib.openid import Server as OpenIDServer
+        from oauthlib.oauth2 import Server as OAuth2Server  # noqa: PLC0415
+        from oauthlib.openid import Server as OpenIDServer  # noqa: PLC0415
 
         endpoint = OAuth2Server(None)
         metadata = MetadataEndpoint([endpoint], default_claims)
@@ -97,6 +98,7 @@ class MetadataEndpointTest(TestCase):
             "scopes_supported": ["email", "profile"],
             "grant_types_supported": [
                 "authorization_code",
+                "urn:ietf:params:oauth:grant-type:device_code",
                 "password",
                 "client_credentials",
                 "refresh_token",
@@ -129,9 +131,19 @@ class MetadataEndpointTest(TestCase):
         }
 
         def sort_list(claims):
-            for k in claims.keys():
-                claims[k] = sorted(claims[k])
+            for key, value in claims.items():
+                claims[key] = sorted(value)
 
         sort_list(metadata.claims)
         sort_list(expected_claims)
         self.assertEqual(sorted(metadata.claims.items()), sorted(expected_claims.items()))
+
+    def test_metadata_validate_issuer(self):
+        with self.assertRaises(ValueError):
+            endpoint = TokenEndpoint(
+                None, None, grant_types={"password": None},
+            )
+            metadata = MetadataEndpoint([endpoint], {
+                "issuer": 'http://foo.bar',
+                "token_endpoint": "https://foo.bar/token",
+            })
