@@ -3,17 +3,16 @@
 Fairly trivial in all grants except the Authorization Code Grant where scope
 need to be persisted temporarily in an authorization code.
 """
-from __future__ import absolute_import, unicode_literals
-
 import json
+from unittest import mock
 
-import mock
+from oauthlib.oauth2 import (
+    BackendApplicationServer, LegacyApplicationServer, MobileApplicationServer,
+    RequestValidator, Server, WebApplicationServer,
+)
 
-from oauthlib.oauth2 import (BackendApplicationServer, LegacyApplicationServer,
-                             MobileApplicationServer, RequestValidator, Server,
-                             WebApplicationServer)
+from tests.unittest import TestCase
 
-from ....unittest import TestCase
 from .test_utils import get_fragment_credentials, get_query_credentials
 
 
@@ -65,13 +64,13 @@ class TestScopeHandling(TestCase):
         for scope, correct_scopes in scopes:
             scopes, _ = self.web.validate_authorization_request(
                     uri % (scope, 'code'))
-            self.assertItemsEqual(scopes, correct_scopes)
+            self.assertCountEqual(scopes, correct_scopes)
             scopes, _ = self.mobile.validate_authorization_request(
                     uri % (scope, 'token'))
-            self.assertItemsEqual(scopes, correct_scopes)
+            self.assertCountEqual(scopes, correct_scopes)
             scopes, _ = self.server.validate_authorization_request(
                 uri % (scope, 'code'))
-            self.assertItemsEqual(scopes, correct_scopes)
+            self.assertCountEqual(scopes, correct_scopes)
 
     def test_scope_preservation(self):
         scope = 'pics+http%3A%2f%2fa.b%2fvideos'
