@@ -1,15 +1,12 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
-
 import json
-
-import mock
+from unittest import mock
 
 from oauthlib.common import Request
 from oauthlib.oauth2.rfc6749.grant_types import ClientCredentialsGrant
 from oauthlib.oauth2.rfc6749.tokens import BearerToken
 
-from ....unittest import TestCase
+from tests.unittest import TestCase
 
 
 class ClientCredentialsGrantTest(TestCase):
