@@ -1,22 +1,21 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
-
 import json
-
-import mock
+from unittest import mock
 
 from oauthlib import common
 from oauthlib.oauth2.rfc6749 import errors, tokens
 from oauthlib.oauth2.rfc6749.endpoints import Server
-from oauthlib.oauth2.rfc6749.endpoints.authorization import AuthorizationEndpoint
+from oauthlib.oauth2.rfc6749.endpoints.authorization import (
+    AuthorizationEndpoint,
+)
 from oauthlib.oauth2.rfc6749.endpoints.resource import ResourceEndpoint
 from oauthlib.oauth2.rfc6749.endpoints.token import TokenEndpoint
-from oauthlib.oauth2.rfc6749.grant_types import (AuthorizationCodeGrant,
-                                                 ClientCredentialsGrant,
-                                                 ImplicitGrant,
-                                                 ResourceOwnerPasswordCredentialsGrant)
+from oauthlib.oauth2.rfc6749.grant_types import (
+    AuthorizationCodeGrant, ClientCredentialsGrant, ImplicitGrant,
+    ResourceOwnerPasswordCredentialsGrant,
+)
 
-from ...unittest import TestCase
+from tests.unittest import TestCase
 
 
 class AuthorizationEndpointTest(TestCase):
@@ -73,7 +72,7 @@ class AuthorizationEndpointTest(TestCase):
             uri, scopes=['all', 'of', 'them'])
         self.assertIn('Location', headers)
         self.assertURLEqual(headers['Location'], 'http://back.to/me?state=xyz', parse_fragment=True)
-        self.assertEqual(body, None)
+        self.assertIsNone(body)
         self.assertEqual(status_code, 302)
 
         # and without the state parameter
@@ -83,7 +82,7 @@ class AuthorizationEndpointTest(TestCase):
             uri, scopes=['all', 'of', 'them'])
         self.assertIn('Location', headers)
         self.assertURLEqual(headers['Location'], 'http://back.to/me', parse_fragment=True)
-        self.assertEqual(body, None)
+        self.assertIsNone(body)
         self.assertEqual(status_code, 302)
 
     def test_missing_type(self):
